@@ -1,19 +1,6 @@
-# Licensed to Elasticsearch B.V. under one or more contributor
-# license agreements. See the NOTICE file distributed with
-# this work for additional information regarding copyright
-# ownership. Elasticsearch B.V. licenses this file to you under
-# the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http:www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing,
-# software distributed under the License is distributed on an
-# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-# KIND, either express or implied.  See the License for the
-# specific language governing permissions and limitations
-# under the License.
+# Copyright The OpenTelemetry Authors
+# SPDX-License-Identifier: Apache-2.0
+
 import logging
 import os
 import sys
@@ -45,6 +32,7 @@ outcome = None
 
 
 def pytest_addoption(parser):
+    """Init command line arguments"""
     group = parser.getgroup("pytest-otel", "report OpenTelemetry traces for tests executed.")
 
     group.addoption(
@@ -89,6 +77,7 @@ def pytest_addoption(parser):
 
 
 def init_otel():
+    """Init the OpenTelemetry settings"""
     global tracer, session_name, service_name, insecure, otel_exporter
     LOGGER.debug("Init Otel : {}".format(service_name))
     trace.set_tracer_provider(
@@ -109,6 +98,7 @@ def init_otel():
 
 
 def start_span(span_name, context=None, kind=None):
+    """Starts a span with the name, context, and kind passed as parameters"""
     global tracer, spans
     spans[span_name] = tracer.start_span(
         span_name, context=context, record_exception=True, set_status_on_exception=True,
@@ -119,6 +109,7 @@ def start_span(span_name, context=None, kind=None):
 
 
 def end_span(span_name, outcome):
+    """Ends a span identified by its name"""
     global spans
     status = convertOutcome(outcome)
     spans[span_name].set_status(status)
@@ -129,21 +120,46 @@ def end_span(span_name, outcome):
 
 
 def convertOutcome(outcome):
+    """Convert from pytest outcome to OpenTelemetry status code"""
     if outcome == "passed":
         return Status(status_code=StatusCode.OK)
-    elif outcome == "failed":
+    elif (outcome == "failed"
+            or outcome == "interrupted"
+            or outcome == "internal_error"
+            or outcome == "usage_error"
+            or outcome == "no_tests_collected"
+        ):
         return Status(status_code=StatusCode.ERROR)
     else:
         return Status(status_code=StatusCode.UNSET)
 
 
+def exitCodeToOutcome(exit_code):
+    """convert pytest ExitCode to outcome"""
+    if exit_code == 0:
+        return "passed"
+    elif exit_code == 1:
+        return "failed"
+    elif exit_code == 2:
+        return "interrupted"
+    elif exit_code == 3:
+        return "internal_error"
+    elif exit_code == 4:
+        return "usage_error"
+    elif exit_code == 4:
+        return "no_tests_collected"
+    else:
+        return "failed"
+
 def traceparent_context(traceparent):
+    """Extracts the trace context from the TRACEPARENT passed"""
     carrier = {}
     carrier["traceparent"] = traceparent
     return TraceContextTextMapPropagator().extract(carrier=carrier)
 
 
 def pytest_sessionstart(session):
+    """Uses the commandline parameter to define the environment variables used by OpenTelemetry"""
     global service_name, traceparent, session_name, insecure, in_memory_span_exporter, otel_span_file_output
     LOGGER.setLevel(logging.DEBUG)
     config = session.config
@@ -171,19 +187,22 @@ def pytest_sessionstart(session):
 
 
 def pytest_runtest_setup(item):  # noqa: U100
+    """Clean the global outcome on every test"""
     global outcome
     outcome = None
 
 
 def pytest_report_teststatus(report):
+    """Set the final outcome to the reported outcome"""
     global outcome
     outcome = report.outcome
 
 
 def pytest_sessionfinish(session, exitstatus):  # noqa: U100
+    """Ends the parent Opentelemetry span with the session outcome"""
     global session_name, outcome, in_memory_span_exporter, otel_exporter
     LOGGER.debug("Session transaction Ends")
-    end_span(session_name, outcome)
+    end_span(session_name, exitCodeToOutcome(exitstatus))
     LOGGER.debug("in_memory_span_exporter {}".format(in_memory_span_exporter))
     if in_memory_span_exporter:
         print()
@@ -216,7 +235,6 @@ def pytest_runtest_call(item):
 
         if hasattr(sys, "last_value") and hasattr(sys, "last_traceback") and hasattr(sys, "last_type"):
             longrepr = ""
-            outcome = ""
             if not isinstance(sys.last_value, _pytest._code.ExceptionInfo):
                 outcome = "failed"
                 longrepr = sys.last_value
@@ -233,7 +251,10 @@ def pytest_runtest_call(item):
             LOGGER.debug("test.last_value {}".format(sys.last_value))
             stack_trace = repr(traceback.format_exception(sys.last_type, sys.last_value, sys.last_traceback))
             span.set_attribute("test.stack_trace", "{}".format(stack_trace))
-            span.set_attribute("test.error", "{}".format(sys.last_value.args[0]))
+            try:
+                span.set_attribute("test.error", "{}".format(sys.last_value.args[0]))
+            except IndexError:
+                span.set_attribute("test.error", "{}".format("unknown"))
             span.set_attribute("test.last_value", "{}".format(sys.last_value))
             span.set_attribute("test.last_type", "{}".format(sys.last_type))
 
